//************************************************************************************************************************** 
//* BlueBase64
//* Copyright (c) 2002 Josh Harler
//* 
//* This software is provided 'as-is', without any express or implied warranty. In no event
//* will the authors be held liable for any damages arising from the use of this software.
//* 
//* Permission is granted to anyone to use this software for any purpose, including commercial
//* applications, and to alter it and redistribute it freely, subject to the following restrictions:
//* 
//* 	1. The origin of this software must not be misrepresented; you must not claim that you
//* 	wrote the original software. If you use this software in a product, an acknowledgment in the
//* 	product documentation would be appreciated but is not required.
//* 
//* 	2. Altered source versions must be plainly marked as such, and must not be misrepresented as
//* 	being the original software.
//* 
//* 	3. This notice may not be removed or altered from any source distribution.
//*
//*
//* file   Base64DecodeFilter.cpp
//**

// Private Headers =========================================================================================================

// matching header
#include "Base64DecodeFilter.h"


// Private Defines/Enums/Typedefs/Etc ======================================================================================

// Private Classes/Structs =================================================================================================

// Private Global Variables ================================================================================================

// External Global Variables ===============================================================================================

// Private Functions =======================================================================================================

// Functions ===============================================================================================================

Base64DecodeFilter::Base64DecodeFilter()
{
	reset();
}

// -------------------------------------------------------------------------------------------------------------------------

Base64DecodeFilter::~Base64DecodeFilter()
{
}

// -------------------------------------------------------------------------------------------------------------------------

bool Base64DecodeFilter::canRead() const
{
	return (m_inputCnt == 4);
}

// -------------------------------------------------------------------------------------------------------------------------

void Base64DecodeFilter::write( uint8_t byte )
{
	if( byte >= 'A' && byte <= 'Z' ) byte = byte - 'A'; else
	if( byte >= 'a' && byte <= 'z' ) byte = byte - 'a' + 26; else
	if( byte >= '0' && byte <= '9' ) byte = byte - '0' + 52; else
	if( byte == '+'                ) byte = 62; else
	if( byte == '/'                ) byte = 63; else
	if( byte == '='                ) byte = 64;
	else byte = 65;

	if( byte < 65 ) {
		if( byte == 64 ) {
			++m_outputSkip;
			m_input[m_inputCnt++] = 0;
		}
		else {
			m_input[m_inputCnt++] = byte;
		}
		if( m_inputCnt == 4 ) {
			process();
		}
	}
}

// -------------------------------------------------------------------------------------------------------------------------

uint8_t Base64DecodeFilter::read()
{
	uint8_t retVal = m_output[m_outputCnt++];

	if( m_outputCnt == 3 - m_outputSkip ) {
		reset();
	}

	return (retVal);
}

// -------------------------------------------------------------------------------------------------------------------------

void Base64DecodeFilter::streamClosed()
{
	if( m_outputCnt != 0 || m_inputCnt != 0 ) {
		throw Exception("Unexpected end of base64 encoded data.  Decoded data may be corrupt.");
	}
}

// -------------------------------------------------------------------------------------------------------------------------

void Base64DecodeFilter::reset()
{
	m_inputCnt   = 0;
	m_outputCnt  = 0;
	m_outputSkip = 0;
}

// -------------------------------------------------------------------------------------------------------------------------

void Base64DecodeFilter::process()
{
	m_output[0] = ( (m_input[0]       ) << 2) | ((m_input[1] & 0x30) >> 4);
	m_output[1] = ( (m_input[1] & 0x0f) << 4) | ((m_input[2] & 0x3c) >> 2);
	m_output[2] = ( (m_input[2] & 0x03) << 6) | ((m_input[3] & 0x3f)     );

	m_outputCnt = 0;
}
