//************************************************************************************************************************** 
//* BlueBase64
//* Copyright (c) 2002 Josh Harler
//* 
//* This software is provided 'as-is', without any express or implied warranty. In no event
//* will the authors be held liable for any damages arising from the use of this software.
//* 
//* Permission is granted to anyone to use this software for any purpose, including commercial
//* applications, and to alter it and redistribute it freely, subject to the following restrictions:
//* 
//* 	1. The origin of this software must not be misrepresented; you must not claim that you
//* 	wrote the original software. If you use this software in a product, an acknowledgment in the
//* 	product documentation would be appreciated but is not required.
//* 
//* 	2. Altered source versions must be plainly marked as such, and must not be misrepresented as
//* 	being the original software.
//* 
//* 	3. This notice may not be removed or altered from any source distribution.
//*
//*
//* file   Base64EncodeFilter.cpp
//**

// Private Headers =========================================================================================================

// matching header
#include "Base64EncodeFilter.h"


// Private Defines/Enums/Typedefs/Etc ======================================================================================

// Private Classes/Structs =================================================================================================

// Private Global Variables ================================================================================================

// External Global Variables ===============================================================================================

// Private Functions =======================================================================================================

// Functions ===============================================================================================================

Base64EncodeFilter::Base64EncodeFilter() :FilterChunk(3, 4)
{
}

// -------------------------------------------------------------------------------------------------------------------------

Base64EncodeFilter::~Base64EncodeFilter()
{
}

// -------------------------------------------------------------------------------------------------------------------------

void Base64EncodeFilter::processChunk( uint8_t* chunkIn, int chunkInSize, uint8_t* chunkOut, int& )
{
	static const char base64Values[64] = {
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 
		'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 
		'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'
		};

	chunkOut[0] = base64Values[   (chunkIn[0] & 0xfc) >> 2 ];
	chunkOut[1] = base64Values[ ( (chunkIn[0] & 0x03) << 4 ) | ( (chunkIn[1] & 0xf0) >> 4) ];
	chunkOut[2] = base64Values[ ( (chunkIn[1] & 0x0f) << 2 ) | ( (chunkIn[2] & 0xc0) >> 6) ];
	chunkOut[3] = base64Values[   (chunkIn[2] & 0x3f) ];

	int skip = 3 - chunkInSize;
	if( skip != 0 && skip != 3 ) {
		if( skip >= 1 ) chunkOut[3] = '=';
		if( skip >= 2 ) chunkOut[2] = '=';
	}
}
